/*
Esempio NodeMCU con Controllo relè via MQTT
con Feedback e pulsante esterno

Questo esempio fa parte della guida di HTML.it "Domotica per Makers"
consultabile all'indirizzo:
https://www.html.it/guide/domotica-per-makers/

L'esempio è realizzato da Giovanni Bernardo (https://www.twitter.com/settorezero)

PREREQUISITI:
- Libreria PubSubClient by Nick O'Leary
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// *** INIZIO SETUP

// IMPOSTAZIONI RETE WI-FI
const char* ssid = "[SSID]"; // nome della rete Wi-Fi
const char* password = "[PASSWORD]"; // password della rete Wi-Fi
IPAddress ip(192,168,1,134); // indirizzo IP da assegnare a questo dispositivo
IPAddress gateway(192,168,1,1); // indirizzo IP del tuo router/access-point Wi-Fi
IPAddress subnet(255,255,255,0); // subnet mask

// IMPOSTAZIONI PER LA COMUNICAZIONE MQTT
const char* mqtt_server = "192.168.1.101"; // Indirizzo IP del tuo server MQTT
const char* mqtt_username = "[MQTT_USERNAME]"; // Nome utente da utilizzare per l'accesso al broker MQTT (metti "" se non hai impostato utente/password)
const char* mqtt_password = "[MQTT_PASSWORD]"; // Password del tuo server MQTT (metti "" se non hai impostato utente/password)
const uint16_t mqtt_port = 1883; // Porta del tuo server MQTT (default: 1883)
const char* mqtt_client_id = "NodeMCU_Relay_Feedback"; // Nome univoco a piacere da assegnare a questo dispositivo (due dispositivi sulla rete non possono avere lo stesso nome!)
const char* mqtt_topic_set = "esempio/relay/set"; // Topic sul quale ricevere il messaggio per accendere/spegnere il relè
const char* mqtt_topic_stat = "esempio/relay/status"; // Topic sul quale pubblicare il messaggio per segnalare relè acceso/spento

WiFiClient wifiClient;
PubSubClient MQTTClient(mqtt_server, mqtt_port, wifiClient);
bool relayStatus=false; // tiene memoria dello stato attuale del relè. false=spento, true=acceso

#define WIFILED   2  // D4, led blu su nodemcu, attivo basso
#define RELAY     12 // D6, relè attivo alto
#define BUTTON    14 // D5 pulsante esterno, attivo basso

// Collegamento Wi-Fi
void WiFi_connect(void)
  {
  digitalWrite(WIFILED,HIGH);
  Serial.print("Connessione alla rete ");
  Serial.println(ssid);
  
  WiFi.config(ip, subnet, gateway);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) 
    {
    Serial.print('.');
    delay(500);
    }
    
  digitalWrite(WIFILED,LOW); // connesso: led acceso
  Serial.println();
  Serial.println("Connesso!");
  Serial.print("Indirizzo IP: ");
  Serial.println(WiFi.localIP());  
  }

// Connessione al broker MQTT
void mqtt_connect(void) 
  {
  Serial.println("Collegamento al broker MQTT");
  while (!MQTTClient.connected()) 
    {
    if (MQTTClient.connect(mqtt_client_id,mqtt_username,mqtt_password)) 
        {
        Serial.println("Connesso");
        // mi sottoscrivo al topic di esempio per il pilotaggio del relè
        MQTTClient.subscribe(mqtt_topic_set);
        } 
    else 
        {
        Serial.print("Tentativo fallito, codice=");
        Serial.println(MQTTClient.state());
        Serial.println("Nuovo tentativo tra 3 secondi");
        delay(3000);
        }
      }
  }

// funzione per impostare il relè e pubblicarne lo stato
void turnRelay(bool stat)
  {
  if (stat==true)
    {
    digitalWrite(RELAY,HIGH);
    Serial.println("Relay ON");
    MQTTClient.publish(mqtt_topic_stat,"true",true); // pubblico che il relè è acceso
    }
  else
    {
    digitalWrite(RELAY,LOW);
    Serial.println("Relay OFF");
    MQTTClient.publish(mqtt_topic_stat,"false",true); // pubblico che il relè è spento  
    }
  relayStatus=stat; // aggiorno il flag che tiene memoria dello stato attuale del relè
  }
  
// procedura che viene richiamata su ricezione messaggio mqtt
void callback(char* topic, byte* payload, unsigned int length) 
  {
  // scrivo sulla seriale il topic e il contenuto del messaggio
  Serial.print("Messaggio arrivato [");
  Serial.print(topic);
  Serial.print("] Payload=");
  for (int i = 0; i < length; i++) 
    {
    Serial.print((char)payload[i]);
    }
  Serial.println();

  // il messaggio contiene la stringa true? Attivo il relè
  if (compareArrays(payload,"true",length)) 
      {
      turnRelay(true);
      } 
  // il messaggio contiene la stringa false? Disattivo il relè
  if (compareArrays(payload,"false",length))
     {
     turnRelay(false);
     }
  }
  
// semplice funzione per confrontare il contenuto di due array
bool compareArrays(byte* array1, char* array2, uint8_t length)
  {
  for (int i = 0; i < length; i++) 
    {
    if ((char)array1[i]!=array2[i])
      {
      return false;  
      }
    }
  return true;
  }

void setup() 
  {
  Serial.begin(115200);
  pinMode(RELAY,OUTPUT);
  pinMode(WIFILED,OUTPUT); 
  pinMode(BUTTON,INPUT_PULLUP);
  digitalWrite(RELAY,LOW); // relay spento
  WiFi_connect(); // connessione al Wi-Fi
  mqtt_connect(); // connessione al broker MQTT
  MQTTClient.setCallback(callback); // impostazione callback su ricezione messaggio
  MQTTClient.publish(mqtt_topic_stat,"false",true); // pubblico il primo messaggio di relè spento
  }

// main
void loop() 
  {
  if (digitalRead(BUTTON)==LOW) // pulsante premuto
    {
    relayStatus=!relayStatus; // inverto lo stato del relè
    turnRelay(relayStatus); // eseguo il toggle del relè
    delay(250); // attendo 250 millisecondi come funzione anti-rimbalzo
    }
  MQTTClient.loop(); // Questa funzione è importante, va sempre richiamata
  }
